/* bkmkfile.h
   function headers and parameters for bkmkfile.c */

     /*---------------------------------------------------------------*/
     /* Xgopher        version 1.2     20 November 1991               */
     /*                version 1.1     20 April 1991                  */
     /*                version 1.0     04 March 1991                  */
     /* X window system client for the University of Minnesota        */
     /*                                Internet Gopher System.        */
     /* Allan Tuchman, University of Illinois at Urbana-Champaign     */
     /*                Computing and Communications Services Office   */
     /* Copyright 1992 by                                             */
     /*           the Board of Trustees of the University of Illinois */
     /* Permission is granted to freely copy and redistribute this    */
     /* software with the copyright notice intact.                    */
     /*---------------------------------------------------------------*/


#ifndef BKMKFILE_H
#define BKMKFILE_H

#include "gopher.h"

/* string to append as suffix to old xgopher bookmark file */

#define OLD_SUFFIX		"~"

#define USER_STUFF_LEN		5120


void setBkmkFile(
#ifdef PROTO
	char *	/* filename */
#endif
);

void setBkmkAppend(
#ifdef PROTO
	BOOLEAN /* appendFlag */
#endif
);

int bkmkOpen(
#ifdef PROTO
	int	/* flags */
#endif
);

void bkmkSave(
);

void bkmkLoad(
);

static gopherItemP readSavedItem(
#ifdef PROTO
	int,	/* fd 	*/
	char *,	/* defaultHost */
	int	/* defaultPort 	*/
#endif
);
#endif /* BKMKFILE_H */
