/* item.h
   header file for item.c */

     /*---------------------------------------------------------------*/
     /* Xgopher        version 1.2     20 November 1991               */
     /*                version 1.1     20 April 1991                  */
     /*                version 1.0     04 March 1991                  */
     /* X window system client for the University of Minnesota        */
     /*                                Internet Gopher System.        */
     /* Allan Tuchman, University of Illinois at Urbana-Champaign     */
     /*                Computing and Communications Services Office   */
     /* Copyright 1992 by                                             */
     /*           the Board of Trustees of the University of Illinois */
     /* Permission is granted to freely copy and redistribute this    */
     /* software with the copyright notice intact.                    */
     /*---------------------------------------------------------------*/


#ifndef ITEM_H
#define ITEM_H

#include "gopher.h"


gopherItemP	newItem(
		);

gopherItemP	copyItem(
#ifdef PROTO
			gopherItemP	/* gi */
#endif
		);

void		freeItem(
#ifdef PROTO
			gopherItemP	/* gi */
#endif
		);

BOOLEAN		compareItem(
#ifdef PROTO
			gopherItemP,	/* g1 */
			gopherItemP	/* g2 */
#endif
		);

void		initItemFields(
#ifdef PROTO
			gopherItemP	/* gi */
#endif
		);

char *		getItemIndex(
#ifdef PROTO
			gopherItemP	/* gi */
#endif
);

#ifdef DEBUG_LIST
void			printItem(
#ifdef PROTO
			gopherItemP,	/* gi */
			char *		/* label */
#endif
);
#endif /* DEBUG_LIST */

#endif /* ITEM_H */
