/* site.def

   Configuration file that will allow Sun's imake to produce a reasonable
   Makefile under OpenWindows 3.  

   To use this file, use a command such as:

       imake -DUseInstalled -I. $OPENWINHOME/lib/config

   Do not use xmkmf; it will point at possibly non-existant X11R5
   (not OpenWindows) files.

   This file may not be complete.  My aim is not to fix what Sun
   did wrong, but rather to provide something that as least ought to
   work.

   If Sun ever fixes the version of the Imake templates under
   $OPENWINHOME/lib/config to properly build a OW3 Makefile, then
   this file can be scrapped.
*/

#define IncRoot		$(OPENWINHOME)/include
#define UsrLibDir	$(OPENWINHOME)/lib
#define BinDir		$(OPENWINHOME)/bin
#define LibDir		$(OPENWINHOME)/lib

#define ManSuffix	n		/* use just one tab or cpp will die */
