/* markList.h
   header file for markList.c */

     /*---------------------------------------------------------------*/
     /* Xgopher        version 1.2     20 November 1991               */
     /*                version 1.1     20 April 1991                  */
     /*                version 1.0     04 March 1991                  */
     /* X window system client for the University of Minnesota        */
     /*                                Internet Gopher System.        */
     /* Allan Tuchman, University of Illinois at Urbana-Champaign     */
     /*                Computing and Communications Services Office   */
     /* Copyright 1992 by                                             */
     /*           the Board of Trustees of the University of Illinois */
     /* Permission is granted to freely copy and redistribute this    */
     /* software with the copyright notice intact.                    */
     /*---------------------------------------------------------------*/


#ifndef MARKLIST_H
#define MARKLIST_H

#include "gopher.h"

void		markItem(
#ifdef PROTO
			gopherItemP	/* gi */
#endif
		);

gopherItemP	unmarkItem(
#ifdef PROTO
			gopherItemP	/* gi */
#endif
		);

void		unmarkItemN(
#ifdef PROTO
			int		/* n */
#endif
		);

void		unmarkAllItems(
		);

gopherItemP	getMarkN(
#ifdef PROTO
			int		/* n */
#endif
		);

BOOLEAN		anyMarks(
		);

gopherItemP	firstMark(
		);

int		markListLength(
		);

#endif /* MARKLIST_H */
