/* help.h
   function headers for help functions */

     /*---------------------------------------------------------------*/
     /* Xgopher        version 1.2     20 November 1991               */
     /*                version 1.1     20 April 1991                  */
     /*                version 1.0     04 March 1991                  */
     /* X window system client for the University of Minnesota        */
     /*                                Internet Gopher System.        */
     /* Allan Tuchman, University of Illinois at Urbana-Champaign     */
     /*                Computing and Communications Services Office   */
     /* Copyright 1992 by                                             */
     /*           the Board of Trustees of the University of Illinois */
     /* Permission is granted to freely copy and redistribute this    */
     /* software with the copyright notice intact.                    */
     /*---------------------------------------------------------------*/

void setHelpFile(
#ifdef PROTO
	char *		/* file_name */
#endif
);

#ifdef DEBUG
void printHelpIndex(
);
#endif

void buildHelpIndex(
);

char	* getHelpText(
#ifdef PROTO
	char *,		/* section_name */
	char *		/* title */
#endif
);
