/* osdep.h
   try to isolate operating system and environment dependencies. */

     /*---------------------------------------------------------------*/
     /* Xgopher        version 1.2     20 November 1991               */
     /*                version 1.1     20 April 1991                  */
     /*                version 1.0     04 March 1991                  */
     /* X window system client for the University of Minnesota        */
     /*                                Internet Gopher System.        */
     /* Allan Tuchman, University of Illinois at Urbana-Champaign     */
     /*                Computing and Communications Services Office   */
     /* Copyright 1992 by                                             */
     /*           the Board of Trustees of the University of Illinois */
     /* Permission is granted to freely copy and redistribute this    */
     /* software with the copyright notice intact.                    */
     /*---------------------------------------------------------------*/



#ifndef OSDEP_H
#define OSDEP_H

char	*getenv();

#include <string.h>

/* try to find malloc's decl */

#if defined(_POSIX_SOURCE)
#include <stdlib.h>
#else
#  if !defined(NeXt) && !defined(sequent) && !defined(__bsdi__) && !defined(unix)
#include <malloc.h>
#  else 
char	*malloc();
char	*realloc();
#  endif
#endif

#endif /* OSDEP_H */
